from enum import IntEnum

from inoon_lora_packet.packet import Packet, HexConverter


class EventType(IntEnum):
    high_g = 1,
    collapse = 8


class EventV2Packet(Packet):
    def _field_spec(self):
        return [
            {'name': 'type',
             'bytes': 1,
             'convert': HexConverter.hex_to_uint,
             'restrict': [1, 8]},

            {'name': 'x_count',
             'bytes': 1,
             'convert': HexConverter.hex_to_uint,
             'restrict': None},

            {'name': 'y_count',
             'bytes': '1',
             'convert': HexConverter.hex_to_uint,
             'restrict': None},

            {'name': 'z_count',
             'bytes': '1',
             'convert': HexConverter.hex_to_uint,
             'restrict': None},

            {'name': 'rssi',
             'bytes': 1,
             'convert': HexConverter.hex_to_int8,
             'restrict': None},
        ]

    def __str__(self):
        msg = 'EVENT | '
        if self.type == 1:
            msg += 'High-G'
        elif self.type == 8:
            msg += 'Collapse'
        else:
            msg += '{}'.format(self.type)
        msg += ' | '

        msg += 'Intr. Cnt.: {}, {}, {} | '.format(self.x_count, self.y_count,
                                                  self.z_count)

        msg += 'RSSI: {}'.format(self.rssi)
        return msg

    @classmethod
    def encode(cls, event_type, x_count, y_count, z_count, rssi):
        enc_val = ''
        enc_val += format(event_type, '02x')
        enc_val += format(x_count, '02x')
        enc_val += format(y_count, '02x')
        enc_val += format(z_count, '02x')
        enc_val += HexConverter.int_to_hex(rssi, 1, True)
        return enc_val.lower()



from inoon_lora_packet.packet import (PacketHeader, PacketType,
                                      InvalidPacketError,
                                      NotSupportedPacketError)
from inoon_lora_packet.ack import AckV2Packet
from inoon_lora_packet.alive import AliveV2Packet
from inoon_lora_packet.event import EventV2Packet
from inoon_lora_packet.error import ErrorV2Packet
from inoon_lora_packet.notice import NoticeV2Packet
from inoon_lora_packet.data_log import DataLogV2Packet


class PacketParser():
    @classmethod
    def parse(cls, raw_packet):
        if len(raw_packet) < 16:
            raise InvalidPacketError

        header = PacketHeader(raw_packet)
        raw_payload = raw_packet[16:]
        packet = None

        if header.packet.type == PacketType.alive:
            packet = AliveV2Packet(raw_payload)
        elif header.packet.type == PacketType.notice:
            packet = NoticeV2Packet(raw_payload)
        elif header.packet.type == PacketType.data_log:
            packet = DataLogV2Packet(raw_payload)
        elif header.packet.type == PacketType.event:
            packet = EventV2Packet(raw_payload)
        elif header.packet.type == PacketType.ack:
            packet = AckV2Packet(raw_payload)
        elif header.packet.type == PacketType.error:
            packet = ErrorV2Packet(raw_payload)
        else:
            raise NotSupportedPacketError

        return header, packet



from setuptools import setup


setup_requires = [
]

install_requires = [
]

dependency_links = [
]

setup(
    name='inoon_lora_packet',
    version='0.4.6',
    description='LoRa packet parser for Ino-on, Inc.',
    author='Joonkyo Kim',
    author_email='jkkim@ino-on.com',
    packages=["inoon_lora_packet"],
    include_package_data=True,
    install_requires=install_requires,
    setup_requires=setup_requires,
    dependency_links=dependency_links,
    # scripts=['manage.py'],
    entry_points={
        'console_scripts': [
        ],
        "egg_info.writers": [
            "foo_bar.txt = setuptools.command.egg_info:write_arg",
        ],
    },
)
